import binpi


class BasicStructure:
    le_int = binpi.LEInt()
    le_uint = binpi.LEUInt()
    le_short = binpi.LEShort()
    le_ushort = binpi.LEUShort()
    le_byte = binpi.LEByte()
    le_ubyte = binpi.LEUByte()
    le_float = binpi.LEFloat()
    le_double = binpi.LEDouble()

    be_int = binpi.BEInt()
    be_uint = binpi.BEUInt()
    be_short = binpi.BEShort()
    be_ushort = binpi.BEUShort()
    be_byte = binpi.BEByte()
    be_ubyte = binpi.BEUByte()
    be_float = binpi.BEFloat()
    be_double = binpi.BEDouble()
